/* ----------------------------------------------------------------------------
 *  $RCSfile: basics.c,v $
 * 
 * $Revision: 1.4 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: basics.c,v $
 * Revision 1.4  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.3  1999/08/13 23:50:09  larse
 * Nicer header separation; fixed quirk with printing newlines.
 *
 * Revision 1.2  1999/05/20 21:20:07  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.1  1999/05/18 00:47:23  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: basics.c,v 1.4 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <netdb.h>

#include <sys/param.h>


#define ETHER_ADDR_LEN          6
struct  ether_addr {
        u_char octet[ETHER_ADDR_LEN];
};

/* These should be in net/ethernet.h but aren't (under FreeBSD-2.2.6). */
char *ether_ntoa(struct ether_addr *);
int ether_ntohost(char *, struct ether_addr *);


void print_address(const char *hdr, const char *tag, const u_long *addr)
  {
  iprintf(hdr, "%s%s%s", (tag && *tag ? tag : ""), (tag && *tag ? ": " : ""),
	  addr2ascii(AF_INET, addr, sizeof(u_long), 0));
  if(no_names == 0)
    {
    struct hostent *host = 
      gethostbyaddr((char *)addr, sizeof(u_long), AF_INET);
    if(host) printf(" [%s]", host->h_name);
    }
  }


void print_protocol(const char *hdr, const char *tag, const int protocol)
  {
  iprintf(hdr, "%s%s%hu", (tag && *tag ? tag : ""), (tag && *tag ? ": " : ""),
	  protocol);
  if(no_names == 0)
    {
    struct protoent *proto = getprotobynumber(protocol);
    if(proto) printf(" [%s]", proto->p_name);
    }
  }


void print_port(const char *hdr, const char *tag, const int port, 
		const char *proto)
  {
  iprintf(hdr, "%s%s%hu", (tag && *tag ? tag : ""), (tag && *tag ? ": " : ""),
	  port);
  if(no_names == 0)
    {
    struct servent *serv = getservbyport(port, proto);
    if(serv) printf(" [%s]", serv->s_name);
    }
  }


void print_network(const char *hdr, const char *tag, const u_long *addr)
  {
  iprintf(hdr, "%s%s%s", (tag && *tag ? tag : ""), (tag && *tag ? ": " : ""),
	  addr2ascii(AF_INET, addr, sizeof(u_long), 0));
  if(no_names == 0)
    {
    struct netent *net = getnetbyaddr(*addr, AF_INET);
    if(net) printf(" [%s]", net->n_name);
    }
  }
 
void print_ether_address(const char *hdr, const char *tag, 
			 const u_char *eaddr)
  {
  iprintf(hdr, "%s%s%s", (tag && *tag ? tag : ""), (tag && *tag ? ": " : ""),
	  ether_ntoa((struct ether_addr *)eaddr));
  if(no_names == 0)
    {
    char name[MAXHOSTNAMELEN];
    if(ether_ntohost(name, (struct ether_addr *)eaddr) == 0)
      printf(" [%s]", name);
    }
  }


void print_seperator(void)
  {
  iprintf("------",
	  "------------------------------------------------------------\n");
  }
