/* ----------------------------------------------------------------------------
 *  $RCSfile: dvmrp.c,v $
 * 
 * $Revision: 1.2 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: dvmrp.c,v $
 * Revision 1.2  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.1  1999/05/18 00:47:23  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: dvmrp.c,v 1.2 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


/* local header id */
static const char hdr[] = "DVMRP";


struct dvmrp_hdr
  {
  u_char type;
  u_char code;
  u_short cksum;
  u_short reserved;
  u_char v_minor;
  u_char v_major;
  };

#define DVMRP_LEAF 1
#define DVMRP_PRUNE 2
#define DVMRP_GENID 4
#define DVMRP_MTRACE 8
#define DVMRP_SNMP 16
#define DVMRP_NETMASK 32
#define DVMRP_FLAGS (DVMRP_LEAF|DVMRP_PRUNE|DVMRP_GENID|DVMRP_MTRACE|\
		     DVMRP_SNMP|DVMRP_NETMASK)




void print_dvmrp(void)
  {
  int fl = 0;
  const struct dvmrp_hdr *dh = 
    (const struct dvmrp_hdr *)next(sizeof(struct dvmrp_hdr));

  if(dh == 0) return;
  iprintf(hdr, "code: %hu ", dh->code);
  switch(dh->code)
    {
    case 1: printf("[probe]\n"); break;
    case 2: printf("[report]\n"); break;
    case 3: printf("[ask neighbors]\n"); break;
    case 4: printf("[neighbors]\n"); break;
    case 5: printf("[ask neighbors 2]\n"); break;
    case 6: printf("[neighbors 2]\n"); break;
    case 7: printf("[prune]\n"); break;
    case 8: printf("[graft]\n"); break;
    case 9: printf("[graft ack]\n"); break;
    default:
      unsupported++;
      printf("[*unsupported*]\n");
    }
  iprintf(hdr, "checksum: %hu\n", ntohs(dh->cksum));
  iprintf(hdr, "minor version: %hu\n", dh->v_minor);
  iprintf(hdr, "major version: %hu\n", dh->v_major);
  switch(dh->code)
    {
    case 1: 
      {
      const u_long *lng;
      iprintf(hdr, "capabilities: %hu", ntohs(dh->reserved));
      if(ntohs(dh->reserved) & DVMRP_FLAGS)
	{
	printf(" [");
	if(ntohs(dh->reserved) & DVMRP_LEAF) printf("%sleaf", fl++ ? "|" : "");
	if(ntohs(dh->reserved) & DVMRP_PRUNE) 
	  printf("%sprune", fl++ ? "|" : "");
	if(ntohs(dh->reserved) & DVMRP_GENID) 
	  printf("%sgenid", fl++ ? "|" : "");
	if(ntohs(dh->reserved) & DVMRP_MTRACE) 
	  printf("%smtrace", fl++ ? "|" : "");
	if(ntohs(dh->reserved) & DVMRP_SNMP) printf("%ssnmp", fl++ ? "|" : "");
	if(ntohs(dh->reserved) & DVMRP_NETMASK) 
	  printf("%snetmask", fl++ ? "|" : "");
	printf("]\n");
	}
      else printf("\n");
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      iprintf(hdr, "generation id: %lu\n", ntohl(*lng));
      iprintf(hdr, "neighbors:\n");
      indent_level++;
      while(left() >= sizeof(u_long))
	{
	lng = (u_long *)next(sizeof(u_long));
	if(lng == 0) return;
	print_address(hdr, 0, lng);
	printf("\n");
	}
      indent_level--;
      break;
      }

    case 2: 
      /* at least 3 octets (mask) must be there */
      while(left() >= 3)
	{
	/* get the weird dvmrp 3-byte mask and massage it for printing */
	u_char addr[4];
	int len = 1;
	const u_char *oct = next(sizeof(u_char));

	if(oct == 0) return;
	addr[1] = *oct;
	if(*oct) ++len;
	oct = next(sizeof(u_char));
	if(oct == 0) return;
	addr[2] = *oct;
	if(*oct) ++len;
	oct = next(sizeof(u_char));
	if(oct == 0) return;
	addr[3] = *oct;
	if(*oct) ++len;
	print_address(hdr, "mask", (u_long *)addr);
	printf("\n");
	/* handle special case described in draft-ietf-idmr-dvmrp-v3-08.txt */
	if(addr[1] == addr[2] == addr[3] == 0) addr[0] = 0;
	else addr[0] = 0xff;
	indent_level++;
	/* get var-length source addresses while metric's hi bit is clear */
	/* at least len + 1 bytes must be left (for src addr + metric) */
	while(left() >= len + 1)
	  {
	  int i;
	  /* get var-length source address */
	  addr[0] = addr[1] = addr[2] = addr[3] = 0;
	  for(i = 0; i < len; i++)
	    {
	    oct = next(sizeof(u_char));
	    if(oct == 0) return;
	    addr[i] = *oct;
	    }
	  print_network(hdr, "source network", (u_long *)addr);
	  printf("\n");
	  /* get metric */
	  oct = next(sizeof(u_char));
	  if(oct == 0) return;
	  iprintf(hdr, "metric: %hu [", *oct & 0x7f);
	  if((*oct & 0x7f) >= 1 && (*oct & 0x7f) <= 31) printf("reachable");
	  else if((*oct & 0x7f) == 32) printf("infinity");
	  else if((*oct & 0x7f) >= 33 && (*oct & 0x7f) <= 63)
	    printf("upstream dependent");
	  else if(addr[0] != 0 || addr[1] != 0 || addr[2] != 0 || addr[3] != 0)
	    {
	    unsupported++;
	    printf("*unsupported*");
	    }	    
	  printf("]\n");
	  /* bail if last flag set */
	  if(*oct & 0x80) break;
	  }
	indent_level--;
	}
      break;
      
    case 3:
      break;

    case 4:
      break;

    case 5:
      /* only header */
      break;

    case 6:
      break;

    case 7:
      {
      const u_long *lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "source host address", lng);
      printf("\n");
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "group address", lng);
      printf("\n");
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      iprintf(hdr, "prune lifetime: %lu [seconds]\n", *lng);
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "source network mask", lng);
      printf("\n");
      break;
      }

    case 8: 
    case 9:
      {
      const u_long *lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "source host address", lng);
      printf("\n");
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "group address", lng);
      printf("\n");
      lng = (u_long *)next(sizeof(u_long));
      if(lng == 0) return;
      print_address(hdr, "source network mask", lng);
      printf("\n");
      break;
      }

    default:
      unsupported++;
      printf("[*unsupported*]\n");
    }
  
  }
