/* ----------------------------------------------------------------------------
 *  $RCSfile: ipdump.h,v $
 * 
 * $Revision: 1.13 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:18 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: ipdump.h,v $
 * Revision 1.13  2000/06/07 16:43:18  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.12  1999/08/13 23:50:10  larse
 * Nicer header separation; fixed quirk with printing newlines.
 *
 * Revision 1.11  1999/08/10 22:16:20  larse
 * Preliminary RIP support (still lacking most options.)
 *
 * Revision 1.10  1999/05/20 21:20:09  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.9  1999/05/18 00:47:24  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.8  1999/05/12 13:04:59  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.7  1999/05/11 17:45:06  larse
 * Initial support for IGMPv4, mostly missing is support for DVMRP.
 *
 * Revision 1.6  1999/03/26 00:59:46  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.5  1999/03/25 20:23:03  larse
 * Now printing a note when unsupported packet types are
 * encountered. Libpcap location now configurable, this allows
 * compilation with CAIRN's broken libpcap (fixed one in
 * /home/d7unsupp/FreeBSD). Minor other changes.
 *
 * Revision 1.4  1999/03/24 00:29:57  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * Revision 1.3  1999/03/23 03:07:06  larse
 * Added support for TCP and UDP. Cleaned up IP option handling and ICMP.
 *
 * Revision 1.2  1999/03/22 20:20:22  larse
 * First stable and partially working version. IPv4 and ICMPv4 are the
 * only supported protocols, but they work fine, even with
 * extensions. The reorganized code should allow for much easier
 * extensions for other protocols.
 *
 * Revision 1.1  1999/03/19 04:22:48  larse
 * Initial revision
 *
 * ----------------------------------------------------------------------------
 */


#ifndef _ipdump_h_
#define _ipdump_h_

#include <sys/types.h>
#include <netinet/in.h>

/* global pointer to argv[0], used mostly for error messages */
extern const char *exec_name;

/* global flag indicating an unsupported header element was encountered */
extern u_char unsupported;

/* global flag indicating verbose output (raw payload will be printed) */
extern u_char verbose;

/* global flag indicating that no name lookups should be done */
extern u_char no_names;

/* global flag indicating link-layer headers should be printed */
extern u_char print_ll_hdr;

#if 0
/* dump packets with unsupported fields to stdout */
extern u_char dump_unsupported;
#endif

/* see the C code for documentation on the following functions */
void print_ether(void);
void print_ipv4(const short recurse);
void print_ipv4_content(const u_char protocol, const u_long destination);
void print_icmpv4(void);
void print_igmpv4(void);
void print_udp(const u_long destination);
void print_tcp(void);
void print_esp(void);
void print_ah(const u_long destination);
void print_raw(const char * const hdr, const size_t len);
void print_sap(void);
void print_sdp(void);
void print_dvmrp(void);
void print_address(const char *hdr, const char *tag, const u_long *addr);
void print_protocol(const char *hdr, const char *tag, const int protocol);
void print_port(const char *hdr, const char *tag, const int port, 
		const char *proto);
void print_network(const char *hdr, const char *tag, const u_long *addr);
void print_ether_address(const char *hdr, const char *tag, 
			 const u_char *eaddr);
void print_arp(void);
void print_rip(void);
void print_seperator(void);

#endif
