/* ----------------------------------------------------------------------------
 *  $RCSfile: raw.c,v $
 * 
 * $Revision: 1.4 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:18 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: raw.c,v $
 * Revision 1.4  2000/06/07 16:43:18  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.3  1999/03/27 02:44:19  larse
 * Added support for ICMPv4 type 40 (security failure). CAIRN IPsec does
 * not seem to generate those, however. The -v option is now controls
 * more things and has been debugged. Minor cleanup of the code. Added
 * ipd script as a convenient shortcut, it may go away later.
 *
 * Revision 1.2  1999/03/26 00:59:47  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.1  1999/03/24 00:29:58  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: raw.c,v 1.4 2000/06/07 16:43:18 larse Exp $";


#include <stdio.h>
#include <ctype.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"

#define LINE 8

/* print raw data with header hdr (for iprintf) of len nicely formatted */
void print_raw(const char * const hdr, const size_t len)
  {
  char ascii[LINE + 1];
  size_t l;

  for(l = 0; l < len; l += LINE)
    {
    /* get a chunk of 8 bytes or what is left */
    size_t c;
    size_t chunk = len - l > LINE ? LINE : len - l;
    const u_char *data = next(chunk);

    if(data == 0) return;
    iprintf(hdr, "");
    for(c = 0; c < chunk; c++)
      {
      /* print the chunk in hex */
      printf("%02x%s", data[c], (c + 1) % 4 ? " " : "   ");
      /* translate each byte into ascii */
      if(isprint(data[c])) ascii[c] = data[c];
      else ascii[c] = ' ';
      }
    /* print spaces for last line so indentation is ok */
    while(c < LINE) 
      {
      printf("  %s", (c + 1) % 4 ? " " : "   ");
      ascii[c] = ' ';
      c++;
      }
    ascii[c] = 0;
    /* print line in ascii */
    printf("[%s]\n", ascii);
    }
  }
