/* ----------------------------------------------------------------------------
 *  $RCSfile: scan.c,v $
 * 
 * $Revision: 1.5 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:18 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: scan.c,v $
 * Revision 1.5  2000/06/07 16:43:18  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.4  1999/03/26 00:59:47  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.3  1999/03/24 00:29:58  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * Revision 1.2  1999/03/23 03:07:07  larse
 * Added support for TCP and UDP. Cleaned up IP option handling and ICMP.
 *
 * Revision 1.1  1999/03/22 20:20:23  larse
 * First stable and partially working version. IPv4 and ICMPv4 are the
 * only supported protocols, but they work fine, even with
 * extensions. The reorganized code should allow for much easier
 * extensions for other protocols.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: scan.c,v 1.5 2000/06/07 16:43:18 larse Exp $";


#include <stdio.h>
#include <string.h>

#include "scan.h"
#include "ipdump.h"

/* local state */
static u_long len = 0;
static const u_char *data = 0;
static const u_char *pos = 0;


void init_scanner(const u_long buf_len, const u_char *buf)
  {
  len = buf_len;
  data = buf;
  pos = data;
  }


const u_char *next(const size_t buf_len)
  {
  const u_char *p = pos;

  if(pos >= data + len)
    {
    fprintf(stderr, "%s: packet header truncated after %lu bytes\n", exec_name,
	    position());
    return 0;
    }
  pos += buf_len;
  return p;
  }
  

u_long position(void) { return pos - data; }


u_long left(void) 
  {
  return len - position(); 
  }


void unget(const size_t back) { pos -= back; }
