/* ----------------------------------------------------------------------------
 *  $RCSfile: udp.c,v $
 * 
 * $Revision: 1.10 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:19 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: udp.c,v $
 * Revision 1.10  2000/06/07 16:43:19  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.9  1999/08/13 23:50:10  larse
 * Nicer header separation; fixed quirk with printing newlines.
 *
 * Revision 1.8  1999/08/10 22:15:57  larse
 * Preliminary RIP support (still lacking most options.)
 *
 * Revision 1.7  1999/05/20 21:20:10  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.6  1999/05/18 00:47:26  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.5  1999/05/12 13:05:01  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.4  1999/03/26 00:59:49  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.3  1999/03/25 20:23:04  larse
 * Now printing a note when unsupported packet types are
 * encountered. Libpcap location now configurable, this allows
 * compilation with CAIRN's broken libpcap (fixed one in
 * /home/d7unsupp/FreeBSD). Minor other changes.
 *
 * Revision 1.2  1999/03/24 00:29:59  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * Revision 1.1  1999/03/23 03:07:08  larse
 * Added support for TCP and UDP. Cleaned up IP option handling and ICMP.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: udp.c,v 1.10 2000/06/07 16:43:19 larse Exp $";

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


struct udphdr 
  {
  u_short uh_sport;               /* source port */
  u_short uh_dport;               /* destination port */
  u_short uh_ulen;                /* udp length */
  u_short uh_sum;                 /* udp checksum */
  };


/* local header id */
static const char hdr[] = "UDP";


/* print udp header and data (as much as we've grabbed) */
void print_udp(const u_long destination)
  {
  const struct udphdr *udp;
  static u_long sap_addr = 0;

  /* only do this once */
  if(sap_addr == 0)
    inet_pton(AF_INET, "224.2.127.254", (void *)&sap_addr);

  udp = (const struct udphdr *)next(sizeof(struct udphdr));
  if(udp == 0) return;
  print_port(hdr, "source port", ntohs(udp->uh_sport), "udp");
  printf("\n");
  print_port(hdr, "destination port", ntohs(udp->uh_dport), "udp");
  printf("\n");
  iprintf(hdr, "length: %hu [octet%s]\n", ntohs(udp->uh_ulen),
	  ntohs(udp->uh_ulen) == 1 ? "" : "s");
  iprintf(hdr, "checksum: %hu\n", ntohs(udp->uh_sum));
  /* eventually, print contents of udp packets based on server port */
  if(left())
    {
    /* see if we have an sap packet */
    switch(ntohs(udp->uh_dport))
      {
      case 9875:
	if(memcmp(&destination, &sap_addr, sizeof(struct in_addr)) == 0) 
	  {
	  print_seperator();
	  print_sap();
	  }
	break;
	
      case 520:
	print_seperator();
	print_rip();
	break;

      default:
	/* we have a packet of a yet unsupported type */
	iprintf(hdr, "data: [%hu octet%s]", left(), left() == 1 ? "" : "s");
	if(verbose)
	  {
	  printf("\n");
	  indent_level++;
	  print_raw(hdr, left());
	  indent_level--;
	  }
	else printf(" [omitted]\n");
      }
    }
  }
